@@ -34,7 +34,7 @@ def _pad_spec(fbank, target_length=1024):
 def pad_wav(waveform, segment_length):
     waveform_length = waveform.shape[-1]
     assert waveform_length > 100, "Waveform is too short, %s" % waveform_length
-    if waveform_length == segment_length:
+    if segment_length is None or waveform_length == segment_length:
         return waveform
     elif waveform_length > segment_length:
         return waveform[:segment_length]
